@@ -1,6 +1,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
+#include <string.h>
 #include "Csnprintf.h"
 
 gid_t *
@@ -24,7 +25,7 @@ Cdecode_groups(char *groups, int *nbgids)
 	p = groups;
 	while (1) {
 		gids[i++] = strtol (p, &dp, 10);
-		if (*dp == '\0') break;
+		if ((dp = strchr (dp, ',')) == NULL) break;
 		p = dp + 1;
 	}
 	return (gids);
