@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: dpm-drain.c,v $ $Revision: 4360 $ $Date: 2011-05-19 13:04:02 +0200 (Thu, 19 May 2011) $ CERN IT-GD/SC Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: dpm-drain.c,v $ $Revision: 10731 $ $Date: 2014-07-02 08:56:04 +0200 (Wed, 02 Jul 2014) $ CERN IT-GD/SC Jean-Philippe Baud";
 #endif /* not lint */
 
 /*      dpm-drain - drain a component of the Light Weight Disk Pool Manager */
@@ -154,9 +154,15 @@ char **argv;
 	int thread_index;
 	char u64buf[21];
 
+	/* fix for bug LCGDM-1332 */
+	setenv("GLOBUS_THREAD_MODEL", "pthread", 1);
+
 	Copterr = 1;
 	Coptind = 1;
 	jid = getpid();
+
+	Cns_set_selectsrvr (CNS_SSRV_NOTPATH);
+
 	while ((c = Cgetopt_long (argc, argv, "", longopts, NULL)) != EOF) {
 		switch (c) {
 		case OPT_FS:
@@ -382,8 +388,13 @@ char **argv;
 				    !strcmp (fs_list[i].fs, lp->fs))
 					break;
 			}
-			if (i >= found_fs)
+			if (i >= found_fs) {
+				if (!fs && !poolname && !strcmp (server, lp->host)) {
+					drainlogit ("Filesystem %s on %s is not managed by DPM, ignoring file %s\n",
+					    lp->fs, lp->host, lp->sfn);
+				}
 				continue;
+			}
 
 			while (! nbfree) {
 				if (Cthread_cond_wait (&condvar) < 0) {
@@ -495,7 +506,7 @@ void *arg;
 	}
 	current_time = time (0);
 	if (lp->ptime > current_time) {	/* file is pinned */
-		drainlogit ("%s pinned until %s", lp->sfn, ctime (&lp->ptime));
+		drainlogit ("%s pinned until %s", lp->sfn, ctime_r (&lp->ptime, buf));
 		if (min_size == 0)
 			thread_rc = 1;
 		goto end_proc;
@@ -542,18 +553,17 @@ void *arg;
 			sprintf (buf, "'%c'",lp->f_type);
 			break;
 	}
-	drainlogit ("file type:\t%s", buf);
 	if (lp->f_type == 'P')
-		drainlogit (" (does not expire)\n");
+		strcat (buf, " (does not expire)");
 	else {
-		buf[0] = '\0';
-		if ((p = ctime (&rep_entries[i].ltime)))
-			strcpy (buf, p);
-		if ((p = strchr (buf, '\n')))
+		char buf2[26];
+		ctime_r (&rep_entries[i].ltime, buf2);
+		if ((p = strchr (buf2, '\n')))
 			*p = '\0';
-		drainlogit (" (%s on %s)\n",
-		    (rep_entries[i].ltime <= current_time) ? "expired" : "expires", buf);
+		sprintf (&buf[strlen (buf)], ", %s at %s",
+		    (rep_entries[i].ltime <= current_time) ? "expired" : "expires", buf2);
 	}
+	drainlogit ("file type:\t%s\n", buf);
 	drainlogit ("size:\t\t%s\n", u64tostru (statbuf.filesize, u64buf, 0));
 	if (rep_entries[i].setname == NULL || *rep_entries[i].setname == '\0')
 		drainlogit ("space:\t\tnot in any space\n");
