@@ -4,13 +4,16 @@
  */
  
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: dpm-ping.c,v $ $Revision: 3960 $ $Date: 2010-11-11 11:30:15 +0100 (Thu, 11 Nov 2010) $ CERN IT-GD/ITR Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: dpm-ping.c,v $ $Revision: 9750 $ $Date: 2013-11-28 11:04:23 +0100 (Thu, 28 Nov 2013) $ CERN IT-GD/ITR Jean-Philippe Baud";
 #endif /* not lint */
 
 /*	dpm-ping - check server alive and return version number */
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
+#if !defined(_WIN32)
+#include <unistd.h>
+#endif
 #include "dpm_api.h"
 #include "serrno.h"
 extern	char	*optarg;
