@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: dpm_main.c,v $ $Revision: 8697 $ $Date: 2013-04-22 21:46:16 +0200 (Mon, 22 Apr 2013) $ CERN IT-GD/CT Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: dpm_main.c,v $ $Revision: 10850 $ $Date: 2014-07-30 14:45:25 +0200 (Wed, 30 Jul 2014) $ CERN IT-GD/CT Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <signal.h>
@@ -175,8 +175,10 @@ u_signed64 *num_deleted;
 						bol2 = 0;
 						if (pfr_entry.status == DPM_READY ||
 						    pfr_entry.status == DPM_RUNNING) {
-							deleted = 0;
-							break;
+							if (* pfr_entry.pfn) {
+								deleted = 0;
+								break;
+							}
 						}
 					}
 					(void) dpm_list_pfr_entry (dbfd, bol2, dpm_req.r_token,
@@ -290,6 +292,8 @@ struct main_args *main_args;
 	}
 
 	dpmlogit (func, "started (DPM %s-%d)\n", BASEVERSION, PATCHLEVEL);
+	dpmlogit (func, "Fast threads: %d, Slow threads: %d\n", nbfthreads, nbsthreads);
+
 	gethostname (localhost, CA_MAXHOSTNAMELEN+1);
 	if (Cdomainname (localdomain, sizeof(localdomain)) < 0) {
 		dpmlogit (func, "Unable to get local domain name\n");
@@ -300,10 +304,14 @@ struct main_args *main_args;
 		strcat (localhost, localdomain);
 	}
 
+	dpmlogit (func, "Local host: %s\n", localhost);
+
 	if ((p = getenv ("DPNS_HOST")))
 		strcpy (dpnshost, p);
 	dpmlogit (func, "DPNS_HOST = %s\n", dpnshost);
 
+	Cns_set_selectsrvr (CNS_SSRV_NOTPATH);
+
 	if ((pw = Cgetpwnam (STAGERSUPERUSER)) == NULL) {
 		dpmlogit (func, "%s account is not defined in passwd file\n",
 		    STAGERSUPERUSER);
@@ -351,9 +359,12 @@ struct main_args *main_args;
 	(void) dpm_init_dbpkg ();
 	memset (&dbfd, 0, sizeof(dbfd));
 	dbfd.idx = nbfthreads + nbsthreads + 1;
-	if (dpm_opendb (&dbfd) < 0)
+	if (dpm_opendb (&dbfd) < 0) {
+		dpmlogit (func, "Error accessing database.\n");
 		return (SYERR);
+	}
 
+	dpmlogit (func, "DB connection looks OK.\n");
 	/* Get current disk pool configuration */
 
 	if (dpm_getpoolconf (&dbfd)) {
@@ -363,7 +374,9 @@ struct main_args *main_args;
 
 	/* Recover the queue of uncompleted requests */
 
+	dpmlogit (func, "Recovering queue...\n");
 	c = dpm_recover_queue (&dbfd);
+	dpmlogit (func, "Finished recovering queue.\n");
 
 	/* Reallocate space for put requests in active/running state */
 
@@ -485,7 +498,7 @@ struct main_args *main_args;
                 }
                 listen_socks[num_listen_socks] = s;
                 ++num_listen_socks;
-                listen (s, 5);
+                listen (s, 127);
         }
         freeaddrinfo (aitop);
                                         
@@ -590,6 +603,7 @@ doit(arg)
 	int magic;
 	int nbfqans = 0;
 	char *req_data;
+	size_t req_datalen;
 	int req_type = 0;
 	char reqbuf[REQBUFSZ-3*LONGSIZE];
 	int set_authorization_id = 1;
@@ -682,6 +696,8 @@ doit(arg)
 			thip->s = -1;
 			return NULL;
 		}
+		if (voname && fqan && nbfqans)
+			Cns_client_setVOMS_data (voname, fqan, 1);
 		if (Cns_getidmapc (thip->Csec_auth_id, user_ca, nbfqans,
 		    (const char **)fqan, &thip->Csec_uid, thip->Csec_gids)) {
 			if (serrno != EACCES)
@@ -706,8 +722,13 @@ doit(arg)
 		Cns_client_resetAuthorizationId ();
 #endif
 	req_data = reqbuf;
-	if ((c = getreq (thip->s, &magic, &req_type, &req_data, &clienthost)) == 0) {
-		procreq (magic, req_type, req_data, clienthost, thip);
+	if ((c = getreq (thip->s, &magic, &req_type, &req_data, &req_datalen,
+	    &clienthost)) == 0) {
+		procreq (magic, req_type, req_data, req_datalen, clienthost, thip);
+		if (thip->dbfd.tr_started) {
+			(void) dpm_end_tr (&thip->dbfd);
+			dpmlogit (func, "Ended an unexpected database transaction\n");
+		}
 		if (req_data != reqbuf)
 			free (req_data);
 	} else {
@@ -826,11 +847,12 @@ gcthread(void *arg)
 	return (NULL);
 }
 
-getreq(s, magic, req_type, req_data, clienthost)
+getreq(s, magic, req_type, req_data, req_datalen, clienthost)
 int s;
 int *magic;
 int *req_type;
 char **req_data;
+size_t *req_datalen;
 const char **clienthost;
 {
 	int l;
@@ -868,6 +890,7 @@ const char **clienthost;
 					return (SEINTERNAL);
 				}
 			}
+			*req_datalen = l;
 			return (0);
 		}
 		l = n;
@@ -902,10 +925,11 @@ inc_reqctr()
 	return (0);
 }
 
-procreq(magic, req_type, req_data, clienthost, thip)
+procreq(magic, req_type, req_data, req_datalen, clienthost, thip)
 int magic;
 int req_type;
 char *req_data;
+size_t req_datalen;
 const char *clienthost;
 struct dpm_srv_thread_info *thip;
 {
@@ -937,103 +961,107 @@ struct dpm_srv_thread_info *thip;
 	}
 	switch (req_type) {
 	case DPM_ABORTFILES:
-		c = dpm_srv_abortfiles (magic, req_data, clienthost, thip);
+		c = dpm_srv_abortfiles (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_ABORTREQ:
-		c = dpm_srv_abortreq (magic, req_data, clienthost, thip);
+		c = dpm_srv_abortreq (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_ADDFS:
-		c = dpm_srv_addfs (magic, req_data, clienthost, thip);
+		c = dpm_srv_addfs (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_ADDPOOL:
-		c = dpm_srv_addpool (magic, req_data, clienthost, thip);
+		c = dpm_srv_addpool (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_COPY:
-		c = dpm_srv_copy (magic, req_data, clienthost, thip);
+		c = dpm_srv_copy (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_DELREPLICA:
-		c = dpm_srv_delreplica (magic, req_data, clienthost, thip);
+		c = dpm_srv_delreplica (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_EXTENDLIFE:
-		c = dpm_srv_extendfilelife (magic, req_data, clienthost, thip);
+		c = dpm_srv_extendfilelife (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_GET:
-		c = dpm_srv_get (magic, req_data, clienthost, thip);
+		c = dpm_srv_get (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_GETPOOLFS:
-		c = dpm_srv_getpoolfs (magic, req_data, clienthost, thip);
+		c = dpm_srv_getpoolfs (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_GETPOOLS:
-		c = dpm_srv_getpools (magic, req_data, clienthost, thip);
+		c = dpm_srv_getpools (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_GETPROTO:
-		c = dpm_srv_getprotocols (magic, req_data, clienthost, thip);
+		c = dpm_srv_getprotocols (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_GETREQID:
-		c = dpm_srv_getreqid (magic, req_data, clienthost, thip);
+		c = dpm_srv_getreqid (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_GETREQSUM:
-		c = dpm_srv_getreqsummary (magic, req_data, clienthost, thip);
+		c = dpm_srv_getreqsummary (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_GETSPACEMD:
-		c = dpm_srv_getspacemd (magic, req_data, clienthost, thip);
+		c = dpm_srv_getspacemd (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_GETSPACETKN:
-		c = dpm_srv_getspacetoken (magic, req_data, clienthost, thip);
+		c = dpm_srv_getspacetoken (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_GETSTSCOPY:
-		c = dpm_srv_getstatus_copyreq (magic, req_data, clienthost, thip);
+		c = dpm_srv_getstatus_copyreq (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_GETSTSGET:
-		c = dpm_srv_getstatus_getreq (magic, req_data, clienthost, thip);
+		c = dpm_srv_getstatus_getreq (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_GETSTSPUT:
-		c = dpm_srv_getstatus_putreq (magic, req_data, clienthost, thip);
+		c = dpm_srv_getstatus_putreq (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_INCREQCTR:
-		c = dpm_srv_inc_reqctr (magic, req_data, clienthost, thip);
+		c = dpm_srv_inc_reqctr (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_MODFS:
-		c = dpm_srv_modifyfs (magic, req_data, clienthost, thip);
+		c = dpm_srv_modifyfs (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_MODPOOL:
-		c = dpm_srv_modifypool (magic, req_data, clienthost, thip);
+		c = dpm_srv_modifypool (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_PING:
-		c = dpm_srv_ping (magic, req_data, clienthost, thip);
+		c = dpm_srv_ping (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_PUT:
-		c = dpm_srv_put (magic, req_data, clienthost, thip);
+	case DPM_PUTX:
+		c = dpm_srv_put (magic, (req_type == DPM_PUTX) ? 1 : 0, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_PUTDONE:
-		c = dpm_srv_putdone (magic, req_data, clienthost, thip);
+		c = dpm_srv_putdone (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_RLSSPACE:
-		c = dpm_srv_releasespace (magic, req_data, clienthost, thip);
+		c = dpm_srv_releasespace (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_RELFILES:
-		c = dpm_srv_relfiles (magic, req_data, clienthost, thip);
+		c = dpm_srv_relfiles (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_RSVSPACE:
-		c = dpm_srv_reservespace (magic, req_data, clienthost, thip);
+		c = dpm_srv_reservespace (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_RM:
-		c = dpm_srv_rm (magic, req_data, clienthost, thip);
+		c = dpm_srv_rm (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_RMFS:
-		c = dpm_srv_rmfs (magic, req_data, clienthost, thip);
+		c = dpm_srv_rmfs (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_RMPOOL:
-		c = dpm_srv_rmpool (magic, req_data, clienthost, thip);
+		c = dpm_srv_rmpool (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_SHUTDOWN:
-		c = dpm_srv_shutdown (magic, req_data, clienthost, thip);
+		c = dpm_srv_shutdown (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_UPDSPACE:
-		c = dpm_srv_updatespace (magic, req_data, clienthost, thip);
+		c = dpm_srv_updatespace (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case DPM_UPDFILSTS:
-		c = dpm_srv_updatefilestatus (magic, req_data, clienthost, thip);
+		c = dpm_srv_updatefilestatus (magic, req_data, req_datalen, clienthost, thip);
+		break;
+	case DPM_ACCESSR:
+		c = dpm_srv_accessr (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	default:
 		sendrep (thip->s, MSG_ERR, DP003, req_type);
@@ -1107,8 +1135,11 @@ void *arg;
 			while (dpm_list_expired_puts (&dbfd, bol, &pfr_entry, 0, &dblistptr) == 0) {
 				bol = 0;
 				if (being_shutdown) break;
-				dpmlogit (func, "removing expired put file: %s\n", pfr_entry.pfn);
-				if (dpm_rm_onereplica (NULL, pfr_entry.pfn,
+				if (* pfr_entry.pfn)
+					dpmlogit (func, "removing expired put file: %s\n", pfr_entry.pfn);
+				else
+					dpmlogit (func, "removing expired put request: %s\n", pfr_entry.r_token);
+				if (!*pfr_entry.pfn || dpm_rm_onereplica (NULL, pfr_entry.pfn,
 				    pfr_entry.requested_size, &status, &dbfd, pfr_entry.s_token,
 				    pfr_entry.actual_size ? 1 : 0, -1, NULL) == 0) {
 					(void) dpm_start_tr (0, &dbfd);
