@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: dpm_mysql_ifce.c,v $ $Revision: 8698 $ $Date: 2013-04-23 09:46:04 +0200 (Tue, 23 Apr 2013) $ CERN IT-GD/CT Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: dpm_mysql_ifce.c,v $ $Revision: 10220 $ $Date: 2014-03-05 17:38:40 +0100 (Wed, 05 Mar 2014) $ CERN IT-GD/CT Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <errno.h>
@@ -48,11 +48,12 @@ unsigned long max;
 
 int dm_mysql_query(MYSQL *mysql, const char *stmt_str) {
 	static char func[] = "dm_mysql_query";
-	int ret;
-	while(1) {
+	int i,ret;
+	for(i=0;i<=MAXRETRY;i++) {
+		if (i>0) { dpmlogit (func,
+			"Restarting query after lock wait timeout\n"); }
 		ret = mysql_query (mysql, stmt_str);
 		if (ret && mysql_errno (mysql) == ER_LOCK_WAIT_TIMEOUT) {
-			dpmlogit (func, "Restarting query after lock wait timeout\n");
 			continue;
 		}
 		break;
@@ -165,6 +166,9 @@ dpm_abort_tr(dbfd)
 struct dpm_dbfd *dbfd;
 {
 	(void) dm_mysql_query (&dbfd->mysql, "ROLLBACK");
+	if (!dbfd->tr_started) {
+		dpmlogit ("dpm_abort_tr", "Aborted transaction when none had been started\n");
+	}
 	dbfd->tr_started = 0;
 	return (0);
 }
@@ -645,6 +649,9 @@ dpm_end_tr(dbfd)
 struct dpm_dbfd *dbfd;
 {
 	(void) dm_mysql_query (&dbfd->mysql, "COMMIT");
+	if (!dbfd->tr_started) {
+		dpmlogit ("dpm_end_tr", "Ended transaction when none had been started\n");
+	}
 	dbfd->tr_started = 0;
 	return (0);
 }
@@ -655,7 +662,10 @@ struct dpm_dbfd *dbfd;
 char *sql_stmt;
 MYSQL_RES **res;
 {
-	while(1) {
+	int i;
+	for(i=0;i<=MAXRETRY;i++) {
+		if (i>0) { dpmlogit (func,
+			"Restarting query after lock wait timeout\n"); }
 		if (dm_mysql_query (&dbfd->mysql, sql_stmt)) {
 			*res = NULL;
 			dpm_mysql_error (func, "mysql_query", dbfd);
@@ -663,7 +673,6 @@ MYSQL_RES **res;
 		}
 		if ((*res = mysql_store_result (&dbfd->mysql)) == NULL) {
 			if (mysql_errno (&dbfd->mysql) == ER_LOCK_WAIT_TIMEOUT) {
-				dpmlogit (func, "Restarting query after lock wait timeout\n");
 				continue;
 			}
 			dpm_mysql_error (func, "mysql_store_res", dbfd);
@@ -671,7 +680,7 @@ MYSQL_RES **res;
 		}
 		break;
 	}
-	return (0);
+	return (*res == NULL) ? -1 : 0;
 }
 
 dpm_get_cpr_by_fullid(dbfd, r_token, f_ordinal, cpr_entry, lock, rec_addr)
