@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: dpm_proclreq.c,v $ $Revision: 8692 $ $Date: 2013-04-22 11:33:34 +0200 (Mon, 22 Apr 2013) $ CERN IT-GD/CT Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: dpm_proclreq.c,v $ $Revision: 9741 $ $Date: 2013-11-27 15:27:33 +0100 (Wed, 27 Nov 2013) $ CERN IT-GD/CT Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <errno.h>
@@ -40,7 +40,7 @@ extern int reqctr;
 extern void *reqctr_lock;
 extern gid_t *Cdecode_groups (char *, int *);
 extern int Cgroupmatch (gid_t, int, gid_t *);
-extern int dpm_rm_onereplica (struct Cns_fileid *, char *, u_signed64, int *, struct dpm_dbfd *dbfd, char *, int, u_signed64, char *);
+extern int dpm_rm_onereplica (struct Cns_filereplicax *, char *, u_signed64, int *, struct dpm_dbfd *dbfd, char *, int, u_signed64, char *);
 
 void *
 msthread(arg)
@@ -193,23 +193,24 @@ void *arg;
 	gids = &dpm_req.r_gid;
 #ifdef CSEC
 	if (dpm_req.r_uid) {
-		Cns_client_setAuthorizationId (dpm_req.r_uid, dpm_req.r_gid,
-		    "GSI", dpm_req.client_dn);
+		Cns_client_resetAuthorizationId();
 #ifdef VIRTUAL_ID
 #ifdef USE_VOMS
 		if ((gids = Cdecode_groups (dpm_req.groups, &nbgids)) == NULL)
-			goto free_voms_info;
+			goto fail_request;
 		alloced_gids = 1;
 		if ((fqan = calloc (nbgids, sizeof(char *))) == NULL)
-			goto free_voms_info;
+			goto fail_request;
 		dpmlogit ("proclreq", "calling Cns_getgrpbygids\n");
 		if (Cns_getgrpbygids (nbgids, gids, fqan) < 0)
-			goto free_voms_info;
+			goto fail_request;
 		if ((voname = strdup (fqan[0])) == NULL)
-			goto free_voms_info;
+			goto fail_request;
 		if ((p = strchr (voname, '/')))
 			*p = '\0';
 
+		Cns_client_setAuthorizationId (dpm_req.r_uid, dpm_req.r_gid,
+		    "GSI", dpm_req.client_dn);
 		/* pass the VOMS auth data to the nameserver */
 		Cns_client_setVOMS_data (voname, fqan, nbgids);
 #endif
@@ -229,6 +230,28 @@ void *arg;
 		dpm_srv_proc_put (&dpm_req, thip, nbgids, gids);
 		break;
 	}
+	goto free_voms_info;
+
+fail_request:
+	(void) dpm_start_tr (thip->s, &thip->dbfd);
+	if (dpm_get_pending_req_by_token (&thip->dbfd, dpm_req.r_token,
+	    &dpm_req, 1, &rec_addr) < 0) {
+		dpm_abort_tr (&thip->dbfd);
+		goto free_voms_info;
+	}
+        if (dpm_delete_pending_entry (&thip->dbfd, &rec_addr) < 0) {
+		dpm_abort_tr (&thip->dbfd);
+		goto free_voms_info;
+	}
+        dpm_req.etime = time (0);
+	dpm_req.status = DPM_FAILED | EINVAL;
+	strcpy (dpm_req.errstring, "Failed to process any of the SURLs");
+	if (dpm_insert_xferreq_entry (&thip->dbfd, &dpm_req) < 0) {
+		dpm_abort_tr (&thip->dbfd);
+		goto free_voms_info;
+	}
+	dpm_end_tr (&thip->dbfd);
+
 free_voms_info:
 	free (voname);
 	if (fqan) {
@@ -316,7 +339,8 @@ dpm_check_replica_online(char *sfn, char *s_token, char *server, char *pfn)
 	}
 	for (i = 0; i < nbreplicas; i++) {
 		if ((rep_entries + i)->status == '-' &&
-		    dpm_get_fs_status ((rep_entries + i)->host, (rep_entries + i)->fs, &status) == 0 &&
+		    dpm_get_fs_status_and_check_pfn ((rep_entries + i)->host,
+		    (rep_entries + i)->fs, (rep_entries + i)->sfn, &status) == 0 &&
 		    !(status & FS_DISABLED)) j++;
 	}
 	if (j == 0) {	/* no valid online replica, check for nearline replica */
@@ -367,7 +391,8 @@ dpm_getbestreplica(char *sfn, char *clienthost, char *server, char *pfn)
 	}
 	for (i = 0; i < nbreplicas; i++) {
 		if ((rep_entries + i)->status == '-' &&
-		    dpm_get_fs_status ((rep_entries + i)->host, (rep_entries + i)->fs, &status) == 0 &&
+		    dpm_get_fs_status_and_check_pfn ((rep_entries + i)->host,
+		    (rep_entries + i)->fs, (rep_entries + i)->sfn, &status) == 0 &&
 		    !(status & FS_DISABLED)) valid_entries[j++] = i;
 	}
 	if (j == 0) {	/* no valid online replica, check for nearline replica */
@@ -406,11 +431,15 @@ dpm_getbestreplica(char *sfn, char *clienthost, char *server, char *pfn)
 		serrno = ENOMEM;
 		return (-1);
 	}
-	p1 = strchr (clienthost, '.') + 1;
-	for (i = 0; i < j; i++) {
-		p2 = strchr ((rep_entries + valid_entries[i])->host, '.') + 1;
-		if (strcmp (p1, p2) == 0)
-			valid_entries1[k++] = valid_entries[i];
+	if ((p1 = strchr (clienthost, '.')) != NULL) {
+		p1++;
+		for (i = 0; i < j; i++) {
+			if ((p2 = strchr ((rep_entries + valid_entries[i])->host, '.')) != NULL) {
+				p2++;
+				if (strcmp (p1, p2) == 0)
+					valid_entries1[k++] = valid_entries[i];
+			}
+		}
 	}
 	if (k) {
 		free (valid_entries);
@@ -635,6 +664,7 @@ gid_t *gids;
 	char pfn[CA_MAXSFNLEN+1];
 	dpm_dbrec_addr rec_addr;
 	dpm_dbrec_addr rec_addrf;
+	char *redir;
 	char server[CA_MAXHOSTNAMELEN+1];
 	char *sfn;
 	struct Cns_filestat st;
@@ -666,6 +696,8 @@ gid_t *gids;
 		(void) dpm_update_gfr_entry (&thip->dbfd, &rec_addrf, &gfr_entry);
 		(void) dpm_end_tr (&thip->dbfd);
 
+		pfn[0] = '\0';
+		server[0] = '\0';
 		if (strncmp (gfr_entry.from_surl, "srm://", 6) == 0) {
 			if ((sfn = sfnfromsurl (gfr_entry.from_surl)) == NULL) {
 				gfr_entry.status = DPM_FAILED | EINVAL;
@@ -688,6 +720,30 @@ gid_t *gids;
 			goto upd_gfr;
 		}
 
+		/* if protocol need redirect with sfn;
+		   replica selection and access time will be handled by redirector
+		*/
+		if (is_redirect_to_headnode_with_sfn (gfr_entry.protocol)) {
+			if ((redir = get_redirect_target (gfr_entry.protocol)) == NULL) {
+				gfr_entry.status = DPM_FAILED | ENOPROTOOPT;
+				goto upd_gfr;
+			}
+			strcpy (gfr_entry.server, redir);
+			gfr_entry.actual_size = st.filesize;
+			current_time = time (0);
+			if (! gfr_entry.lifetime)
+				gfr_entry.lifetime = dpm_get_defpintime ("");
+			if (gfr_entry.lifetime != 0x7FFFFFFF)
+				gfr_entry.lifetime += current_time;
+			maxpintime = dpm_get_maxpintime ("");
+			if (maxpintime != 0x7FFFFFFF)
+				maxpintime += current_time;
+			if (gfr_entry.lifetime > maxpintime)
+				gfr_entry.lifetime = maxpintime;
+			gfr_entry.status = DPM_READY;
+			goto upd_gfr;
+		}
+
 		/* Get best replica */
 
 		if ((c = dpm_getbestreplica (sfn, dpm_req->clienthost, server, pfn)) < 0) {
@@ -737,7 +793,11 @@ gid_t *gids;
 			gfr_entry.status = DPM_FAILED | serrno;
 			goto upd_gfr;
 		}
-		strcpy (gfr_entry.server, server);
+
+		if ((redir = get_redirect_target (gfr_entry.protocol)) != NULL)
+			strcpy (gfr_entry.server, redir);
+		else
+			strcpy (gfr_entry.server, server);
 		strcpy (gfr_entry.pfn, pfn);
 		gfr_entry.actual_size = st.filesize;
 		gfr_entry.status = DPM_READY;
@@ -761,7 +821,7 @@ upd_gfr:
 		}
 		if (gfr_entry.status == DPM_READY)
 			dpmlogit (func, "TURL info: %s %s %s\n",
-			    gfr_entry.protocol, server, pfn);
+			    gfr_entry.protocol, gfr_entry.server, *pfn ? pfn : sfn);
 	}
 
 	if (nb_files_to_recall) {
@@ -834,6 +894,7 @@ gid_t *gids;
 	dpm_dbrec_addr rec_addr;
 	dpm_dbrec_addr rec_addrf;
 	dpm_dbrec_addr rec_addrs;
+	char *redir;
 	struct Cns_filereplicax *rep_entries = NULL;
 	u_signed64 reqsize;
 	gid_t sav_rgid = -1;
@@ -904,6 +965,7 @@ gid_t *gids;
 		file_create = 0;
 		nbreplicas = 0;
 		pfn[0] = '\0';
+		server[0] = '\0';
 		poolname[0] = '\0';
 		rep_entries = NULL;
 		u_space_updated = 0;
@@ -916,6 +978,28 @@ gid_t *gids;
 		} else
 			sfn = pfr_entry.to_surl;
 
+		/* if it is for a headnode with sfn, then
+		   placement for replication isn't supported
+		*/
+		if (is_redirect_to_headnode_with_sfn (pfr_entry.protocol)) {
+			if ((redir = get_redirect_target (pfr_entry.protocol)) == NULL) {
+				pfr_entry.status = DPM_FAILED | ENOPROTOOPT;
+				goto upd_pfr;
+			}
+			if (dpm_req->flags == 4) {
+				pfr_entry.status = DPM_FAILED | EOPNOTSUPP;
+				strcpy (pfr_entry.errstring,
+				    "Replication flag not supported with this protocol");
+				goto upd_pfr;
+			}
+			if (*pfr_entry.server || *pfr_entry.pfn) {
+				pfr_entry.status = DPM_FAILED | EOPNOTSUPP;
+				strcpy (pfr_entry.errstring,
+				    "Specification of server or pfn hint not supported with this protocol");
+				goto upd_pfr;
+			}
+		}
+
 		/* Check if the file exists */
 
 		dpmlogit (func, "calling Cns_stat\n");
@@ -987,7 +1071,6 @@ gid_t *gids;
 					goto upd_pfr;
 				}
 				for (j = 0; j < nbreplicas; j++) {
-					file_uniqueid.fileid = (rep_entries + j)->fileid;
 					reqsize = 0;
 					if ((rep_entries + j)->status == 'P') {
 						(void) dpm_start_tr (thip->s, &thip->dbfd);
@@ -1009,8 +1092,7 @@ gid_t *gids;
 						}
 						(void) dpm_end_tr (&thip->dbfd);
 					}
-					dpm_rm_onereplica (&file_uniqueid,
-					    (rep_entries + j)->sfn, reqsize, &status,
+					dpm_rm_onereplica ((rep_entries + j), NULL, reqsize, &status,
 					    &thip->dbfd, (rep_entries + j)->setname, 0, -1, NULL);
 				}
 				free (rep_entries);
@@ -1059,12 +1141,42 @@ gid_t *gids;
 			}
 			strcpy (poolname, dpm_spcmd.poolname);
 		}
-		 
-		dpmlogit (func, "calling dpm_selectfs\n");
+
+		/* for some protocols placement isn't done now */
+		if (is_redirect_to_headnode_with_sfn (pfr_entry.protocol)) {
+			strcpy (pfr_entry.server, redir);
+			current_time = time (0);
+			if (! pfr_entry.lifetime)
+				pfr_entry.lifetime = dpm_get_defpintime ("");
+			if (pfr_entry.lifetime != 0x7FFFFFFF)
+				pfr_entry.lifetime += current_time;
+			if (* pfr_entry.s_token)
+				maxpintime = dpm_spcmd.expire_time;
+			else {
+				maxpintime = dpm_get_maxpintime ("");
+				if (maxpintime != 0x7FFFFFFF)
+					maxpintime += current_time;
+			}
+			if (pfr_entry.lifetime > maxpintime)
+				pfr_entry.lifetime = maxpintime;
+			pfr_entry.status = DPM_READY;
+			goto upd_pfr;
+		}
+
+		strcpy (server, pfr_entry.server);
+		strcpy (pfn, pfr_entry.pfn);
+		if (*server || *pfn) {
+			dpmlogit (func, "calling dpm_selectfs with requirement server=%s fs=%s\n", server, pfn);
+		} else {
+			dpmlogit (func, "calling dpm_selectfs\n");
+		}
+
 		if (dpm_selectfs (r_gid, pfr_entry.f_type,
 		    pfr_entry.ret_policy, pfr_entry.ac_latency, excl_host, excl_fs,
 		    &pfr_entry.requested_size, server, pfn, poolname, &defpintime) < 0) {
 			pfr_entry.status = DPM_FAILED | ENOSPC;
+			server[0] = '\0';
+			pfn[0] = '\0';
 			goto upd_pfr;
 		}
 		if (excl_mutex_taken) {
@@ -1227,7 +1339,11 @@ gid_t *gids;
 		}
 		if (pfr_entry.lifetime > maxpintime)
 			pfr_entry.lifetime = maxpintime;
-		strcpy (pfr_entry.server, server);
+
+		if ((redir = get_redirect_target (pfr_entry.protocol)) != NULL)
+			strcpy (pfr_entry.server, redir);
+		else
+			strcpy (pfr_entry.server, server);
 		strcpy (pfr_entry.pfn, pfn);
 		if (file_create)
 			pfr_entry.actual_size = LLONG_MAX;
@@ -1244,7 +1360,7 @@ upd_pfr:
 				    &pfr_entry);
 				if (pfr_entry.status == DPM_READY) {
 					/* Add replica entry */
-					if (Cns_addreplicax (NULL, &file_uniqueid, server, pfn, 'P',
+					if (*pfn && Cns_addreplicax (NULL, &file_uniqueid, server, pfn, 'P',
 					    pfr_entry.f_type, poolname, fs,
 					    (dpm_req->flags == 4) ? 'S' : 'P', pfr_entry.s_token) < 0) {
 						pfr_entry.status = DPM_FAILED | serrno;
@@ -1284,7 +1400,7 @@ upd_pfr:
 			nb_file_err++;
 		} else
 			dpmlogit (func, "TURL info: %s %s %s\n",
-			    pfr_entry.protocol, server, pfn);
+			    pfr_entry.protocol, pfr_entry.server, *pfn ? pfn : sfn);
 	}
 
 	/* Move the request to the non-pending queue */
