@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: poolmgr.c,v $ $Revision: 8679 $ $Date: 2013-04-19 09:35:01 +0200 (Fri, 19 Apr 2013) $ CERN IT-GD/CT Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: poolmgr.c,v $ $Revision: 9749 $ $Date: 2013-11-28 09:11:47 +0100 (Thu, 28 Nov 2013) $ CERN IT-GD/CT Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <errno.h>
@@ -269,20 +269,28 @@ dpm_get_def_lifetime(char *pfn, char *poolname)
 
 dpm_get_defpintime(char *pfn)
 {
-	int defpintime = DEFAULT_PINTIME;
+	int defpintime = 0;
 	struct dpm_fs *elemp;
 	int found = 0;
 	int i;
 	int j;
-	char *p;
+	char *p = NULL;
 
-	p = strchr (pfn, ':');
-	if (p == NULL)
-		return (defpintime);	/* cannot find the pool */
+	if (*pfn) {
+		p = strchr (pfn, ':');
+		if (p == NULL)
+			return (DEFAULT_PINTIME);	/* cannot find the pool */
+	}
 	if (Cthread_mutex_lock (&poolconf_mutex) < 0)
-		return (defpintime);
-	*p = '\0';
+		return (DEFAULT_PINTIME);
+	if (p != NULL)
+		*p = '\0';
 	for (i = 0; i < nbpool; i++) {
+		if (! *pfn) {
+			if ((pools + i)->defpintime > defpintime)
+				defpintime = (pools + i)->defpintime;
+			continue;
+		}
 		elemp = (pools + i)->elemp;
 		for (j = 0; j < (pools + i)->nbelem; j++) {
 			if (strcmp (pfn, elemp->server) == 0 &&
@@ -297,7 +305,11 @@ dpm_get_defpintime(char *pfn)
 		if (found) break;
 	}
 	(void) Cthread_mutex_unlock (&poolconf_mutex);
-	*p = ':';
+	if (p != NULL) {
+		*p = ':';
+		if (!found)
+			defpintime = DEFAULT_PINTIME;
+	}
 	return (defpintime);
 }
 
@@ -319,6 +331,75 @@ dpm_get_fs_status (char *server, char *fs, int *status)
 	return (0);
 }
 
+dpm_get_fs_status_and_check_pfn (char *server, char *fs, char *pfn, int *status)
+{
+
+	static char func[] = "dpm_get_fs_status_and_check_pfn";
+	size_t sl, fl, pl;
+	char *p;
+
+	if (dpm_get_fs_status (server, fs, status) < 0) {
+		dpmlogit (func, "No configured filesystem for %s:%s\n",
+		    server, fs);
+		return (-1);
+	}
+
+	fl = strlen (fs);
+	pl = strlen (pfn);
+	sl = 0;
+	if ((p = strchr (pfn, ':')) != NULL)
+		sl = p - pfn;
+
+	if (sl == 0 ||
+	    (!getenv("DPM_IGNORE_PFNSERVERNAME") &&
+	    (sl != strlen(server) || strncmp(server, pfn, sl)))) {
+		dpmlogit (func, "PFN server name invalid: "
+		    "%s (server=%s)\n", pfn, server);
+		serrno = EINVAL;
+		return (-1);
+	}
+
+	if (pl < sl + fl + 3) {
+		dpmlogit (func, "PFN too short to be valid: "
+		    "%s (fs=%s)\n", pfn, fs);
+		serrno = EINVAL;
+		return (-1);
+	}
+
+	if (memcmp (&pfn[sl+1], fs, fl) || pfn[sl+fl+1] != '/') {
+		dpmlogit (func, "PFN filesystem invalid: "
+		    "%s (fs=%s)\n", pfn, fs);
+		serrno = EINVAL;
+		return (-1);
+	}
+
+	if (!getenv("DPM_IGNORE_DOUBLESLASH") &&
+	    strstr (&pfn[sl+fl+1], "//") != NULL) {
+		dpmlogit (func, "PFN contains double slash: %s\n", pfn);
+		serrno = EINVAL;
+		return (-1);
+	}
+
+	if (strstr (&pfn[sl+fl+1], "/../") != NULL ||
+	    !strcmp (&pfn[pl-3], "/..")) {
+		dpmlogit (func, "PFN contains double dot name: "
+		    "%s\n", pfn);
+		serrno = EINVAL;
+		return (-1);
+	}
+
+	if (strstr (&pfn[sl+fl+1], "/./") != NULL ||
+	    !strcmp (&pfn[pl-2], "/.")) {
+		dpmlogit (func, "PFN contains single dot name: "
+		    "%s\n", pfn);
+		serrno = EINVAL;
+		return (-1);
+	}
+
+	return (0);
+}
+
+
 dpm_get_max_lifetime(char *pfn)
 {
 	struct dpm_fs *elemp;
@@ -359,16 +440,24 @@ dpm_get_maxpintime(char *pfn)
 	int found = 0;
 	int i;
 	int j;
-	int maxpintime = MAX_PINTIME;
-	char *p;
+	int maxpintime = 0;
+	char *p = NULL;
 
-	p = strchr (pfn, ':');
-	if (p == NULL)
-		return (0);	/* cannot find the pool */
+	if (*pfn) {
+		p = strchr (pfn, ':');
+		if (p == NULL)
+			return (0);	/* cannot find the pool */
+	}
 	if (Cthread_mutex_lock (&poolconf_mutex) < 0)
-		return (maxpintime);
-	*p = '\0';
+		return (MAX_PINTIME);
+	if (p != NULL)
+		*p = '\0';
 	for (i = 0; i < nbpool; i++) {
+		if (! *pfn) {
+			if ((pools + i)->maxpintime > maxpintime)
+				maxpintime = (pools + i)->maxpintime;
+			continue;
+		}
 		elemp = (pools + i)->elemp;
 		for (j = 0; j < (pools + i)->nbelem; j++) {
 			if (strcmp (pfn, elemp->server) == 0 &&
@@ -383,7 +472,11 @@ dpm_get_maxpintime(char *pfn)
 		if (found) break;
 	}
 	(void) Cthread_mutex_unlock (&poolconf_mutex);
-	*p = ':';
+	if (p != NULL) {
+		*p = ':';
+		if (!found)
+			maxpintime = MAX_PINTIME;
+	}
 	return (maxpintime);
 }
 
@@ -798,7 +891,7 @@ rp_unlock:
 	/* Round-Robin allocation as long as there is enough free space */
 	/* must be at least min_free percent of the capacity remaing if the file was added */
 
-dpm_selectfsinpool(int pool_idx, char **excl_host, char **excl_fs, u_signed64 *size, int *fs_idx, int min_free)
+dpm_selectfsinpool(int pool_idx, char **excl_host, char **excl_fs, u_signed64 *size, int *fs_idx, int min_free, char *required_host, char *required_fs)
 {
 	struct dpm_fs *elemp;
 	int found = 0;
@@ -825,13 +918,16 @@ dpm_selectfsinpool(int pool_idx, char **excl_host, char **excl_fs, u_signed64 *s
 				j++;
 			}
 		}
-		if (! excl_host || ! excl_host[j])
-			if (elemp->status == 0 && elemp->free > 0 &&
-			    elemp->free >= reqsize &&
-			    ((elemp->free - reqsize) * 100) >= (elemp->capacity * min_free)) {
-				found = 1;
-				break;
-			}
+		if (! excl_host || ! excl_host[j]) {
+			if ((!*required_host || !strcmp(required_host, elemp->server)) &&
+			    (!*required_fs || !strcmp(required_fs, elemp->fs)))
+				if (elemp->status == 0 && elemp->free > 0 &&
+				    elemp->free >= reqsize &&
+				    ((elemp->free - reqsize) * 100) >= (elemp->capacity * min_free)) {
+					found = 1;
+					break;
+				}
+		}
 		i++;
 		if (i >= poolp->fs_veclen) i = 0;
 	} while (i != poolp->next_elem);
@@ -865,7 +961,7 @@ dpm_poolmatch (gid_t gid, char f_type, char rp_op, char ret_policy, u_signed64 r
 	return (0);	/* no matching pool */
 }
 
-dpm_poolmatch2(gid_t gid, char f_type, char rp_op, char ret_policy, char **excl_host, char **excl_fs, u_signed64 *size, int *pool_idx, int *fs_idx, int min_free)
+dpm_poolmatch2(gid_t gid, char f_type, char rp_op, char ret_policy, char **excl_host, char **excl_fs, u_signed64 *size, int *pool_idx, int *fs_idx, int min_free, char *required_host, char *required_fs)
 {
 	int i;
 
@@ -875,7 +971,7 @@ dpm_poolmatch2(gid_t gid, char f_type, char rp_op, char ret_policy, char **excl_
 		    ((rp_op == '=' && (pools + i)->ret_policy == ret_policy) ||
 		     (rp_op == '<' && (pools + i)->ret_policy < ret_policy)) &&
 		    (pools + i)->free > 0 &&
-		    dpm_selectfsinpool (i, excl_host, excl_fs, size, fs_idx, min_free) == 0)
+		    dpm_selectfsinpool (i, excl_host, excl_fs, size, fs_idx, min_free, required_host, required_fs) == 0)
 			break;
 	}
 	if (i < nbpool) {
@@ -888,15 +984,61 @@ dpm_poolmatch2(gid_t gid, char f_type, char rp_op, char ret_policy, char **excl_
 dpm_selectfs(gid_t gid, char f_type, char ret_policy, char ac_latency, char **excl_host, char **excl_fs, u_signed64 *size, char *server, char *pfn, char *poolname, int *defpintime)
 {
 	struct dpm_fs *elemp;
+	char *fsp;
 	int i;
 	int ipass;
 	int j;
+	size_t len;
 	int min_free;
+	char *p,*p2;
 	struct dpm_pool *poolp;
+	char required_fs[80];
+	char required_host[CA_MAXHOSTNAMELEN+1];
 	char tmpbuf0[21];
 	char tmpbuf1[21];
 	char tmpbuf2[21];
 
+	if (strlen (server) >= sizeof(required_host)) {
+		/* input server agument too long */
+		serrno = EINVAL;
+		return (-1);
+	} else {
+		strcpy (required_host, server);
+		len = strlen (required_host);
+	}
+
+	p = strchr (pfn, ':');
+	p2 = strchr (pfn, '/');
+
+	if (p && (!p2 || p2>p)) {
+		/* there is a hostname in the pfn hint */
+		if (p - pfn >= sizeof(required_host)) {
+			/* hostname portion of the pfn hint is too long */
+			serrno = EINVAL;
+			return (-1);
+		}
+		if (len && (p - pfn != len || strncmp(required_host, pfn, len))) {
+			/* hostname portion of the pfn hint doesn't match the independently specified required-server */
+			serrno = EINVAL;
+			return (-1);
+		}
+		len = p - pfn;
+		strncpy(required_host, pfn, len);
+		required_host[len] = '\0';
+		fsp = p+1;
+	} else {
+		/* there was no hostname in the pfn hint */
+		fsp = pfn;
+	}
+
+	if (strlen (fsp) >= sizeof(required_fs)) {
+		/* filesystem part of the pfn hint is too long */
+		serrno = EINVAL;
+		return (-1);
+	} else {
+		strcpy(required_fs, fsp);
+	}
+
 	if (Cthread_mutex_lock (&poolconf_mutex) < 0)
 		return (-1);
 
@@ -910,14 +1052,14 @@ dpm_selectfs(gid_t gid, char f_type, char ret_policy, char ac_latency, char **ex
 					min_free = 0;
 					break;
 			}
-			if (dpm_poolmatch2 (gid, f_type, '=', ret_policy, excl_host, excl_fs, size, &i, &j, min_free) ||
-			    dpm_poolmatch2 (gid, f_type, '<', ret_policy, excl_host, excl_fs, size, &i, &j, min_free) ||
-			    dpm_poolmatch2 (gid, '-', '=', ret_policy, excl_host, excl_fs, size, &i, &j, min_free) ||
-			    dpm_poolmatch2 (gid, '-', '<', ret_policy, excl_host, excl_fs, size, &i, &j, min_free) ||
-			    dpm_poolmatch2 (0, f_type, '=', ret_policy, excl_host, excl_fs, size, &i, &j, min_free) ||
-			    dpm_poolmatch2 (0, f_type, '<', ret_policy, excl_host, excl_fs, size, &i, &j, min_free) ||
-			    dpm_poolmatch2 (0, '-', '=', ret_policy, excl_host, excl_fs, size, &i, &j, min_free) ||
-			    dpm_poolmatch2 (0, '-', '<', ret_policy, excl_host, excl_fs, size, &i, &j, min_free)) {
+			if (dpm_poolmatch2 (gid, f_type, '=', ret_policy, excl_host, excl_fs, size, &i, &j, min_free, required_host, required_fs) ||
+			    dpm_poolmatch2 (gid, f_type, '<', ret_policy, excl_host, excl_fs, size, &i, &j, min_free, required_host, required_fs) ||
+			    dpm_poolmatch2 (gid, '-', '=', ret_policy, excl_host, excl_fs, size, &i, &j, min_free, required_host, required_fs) ||
+			    dpm_poolmatch2 (gid, '-', '<', ret_policy, excl_host, excl_fs, size, &i, &j, min_free, required_host, required_fs) ||
+			    dpm_poolmatch2 (0, f_type, '=', ret_policy, excl_host, excl_fs, size, &i, &j, min_free, required_host, required_fs) ||
+			    dpm_poolmatch2 (0, f_type, '<', ret_policy, excl_host, excl_fs, size, &i, &j, min_free, required_host, required_fs) ||
+			    dpm_poolmatch2 (0, '-', '=', ret_policy, excl_host, excl_fs, size, &i, &j, min_free, required_host, required_fs) ||
+			    dpm_poolmatch2 (0, '-', '<', ret_policy, excl_host, excl_fs, size, &i, &j, min_free, required_host, required_fs)) {
 				break;
 			}
 		}
@@ -937,8 +1079,8 @@ dpm_selectfs(gid_t gid, char f_type, char ret_policy, char ac_latency, char **ex
 			serrno = EINVAL;
 			return (-1);
 		}
-		if (dpm_selectfsinpool (i, excl_host, excl_fs, size, &j, 2) < 0 &&
-		    dpm_selectfsinpool (i, excl_host, excl_fs, size, &j, 0) < 0) {
+		if (dpm_selectfsinpool (i, excl_host, excl_fs, size, &j, 2, required_host, required_fs) < 0 &&
+		    dpm_selectfsinpool (i, excl_host, excl_fs, size, &j, 0, required_host, required_fs) < 0) {
 			(void) Cthread_mutex_unlock (&poolconf_mutex);
 			serrno = ENOSPC;
 			return (-1);
