@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: send2dpm.c,v $ $Revision: 4360 $ $Date: 2011-05-19 13:04:02 +0200 (Thu, 19 May 2011) $ CERN IT-GD/CT Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: send2dpm.c,v $ $Revision: 10222 $ $Date: 2014-03-05 22:37:15 +0100 (Wed, 05 Mar 2014) $ CERN IT-GD/CT Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <errno.h>
@@ -15,6 +15,7 @@ static char sccsid[] = "@(#)$RCSfile: send2dpm.c,v $ $Revision: 4360 $ $Date: 20
 #if defined(_WIN32)
 #include <winsock2.h>
 #else
+#include <unistd.h>
 #include <netdb.h>
 #include <netinet/in.h>
 #include <sys/socket.h>
@@ -32,6 +33,35 @@ static char sccsid[] = "@(#)$RCSfile: send2dpm.c,v $ $Revision: 4360 $ $Date: 20
 extern char *ws_strerr;
 #endif
 
+static struct addrinfo *lookup_host(char *func, const char *node, const char *service) {
+	struct addrinfo *aitop;
+	int gaierrno;
+	struct addrinfo hints;
+
+	memset (&hints, 0, sizeof(struct addrinfo));
+	hints.ai_family = PF_UNSPEC;
+	hints.ai_socktype = SOCK_STREAM;
+#ifdef AI_ADDRCONFIG
+	hints.ai_flags |= AI_ADDRCONFIG;
+#endif
+
+	gaierrno = Cgetaddrinfo (node, service, &hints, &aitop);
+
+	if (gaierrno !=0 && serrno == 0)
+		serrno = SENOSHOST;
+
+	if (gaierrno == EAI_NONAME) {
+		dpm_errmsg (func, DP009, "Host unknown:", node);
+		return (NULL);
+	} else if (gaierrno != 0) { 
+		dpm_errmsg (func, "Error during lookup of %s: %s\n",
+				node, Cgai_strerror (gaierrno));
+		return (NULL);
+	}
+
+	return (aitop);
+}
+
 /* send2dpm - send a request to the disk pool manager and wait for the reply */
 
 send2dpm(host, reqp, reql, user_repbuf, user_repbuf_len, repbuf2, nbstruct)
@@ -46,7 +76,7 @@ int *nbstruct;
 	int abortconnect;
 	int actual_replen = 0;
 	struct addrinfo *ai;
-	struct addrinfo *aitop;
+	struct addrinfo *aitop = NULL;
 	int alloced = 0;
 	int c;
 	struct dpm_copyfilestatus *cpr_entry;
@@ -60,11 +90,9 @@ int *nbstruct;
 	char errstring[256];
 	struct dpm_filestatus *filestatus;
 	char func[16];
-	int gaierrno;
 	char *getconfent();
 	char *getenv();
 	struct dpm_getfilestatus *gfr_entry;
-	struct addrinfo hints;
 	int i;
 	int isconnected;
 	int magic;
@@ -113,27 +141,6 @@ int *nbstruct;
 		serrno = 0;
 	}
 
-	memset (&hints, 0, sizeof(struct addrinfo));
-	hints.ai_family = PF_UNSPEC;
-	hints.ai_socktype = SOCK_STREAM;
-#ifdef AI_ADDRCONFIG
-	hints.ai_flags |= AI_ADDRCONFIG;
-#endif
-
-	gaierrno = Cgetaddrinfo (dpmhost, strport, &hints, &aitop);
-
-	if (gaierrno !=0 && serrno == 0)
-		serrno = SENOSHOST;
-
-	if (gaierrno == EAI_NONAME) {
-		dpm_errmsg (func, DP009, "Host unknown:", dpmhost);
-		return (-1);
-	} else if (gaierrno != 0) { 
-		dpm_errmsg (func, "Error during lookup of %s: %s\n",
-				dpmhost, Cgai_strerror (gaierrno));
-		return (-1);
-	}
-
 	/* get retry environment variables */
 	if ((p = getenv ("DPM_CONNTIMEOUT")) == NULL) {
 		timeout = DEFAULT_CONNTIMEOUT;
@@ -158,14 +165,18 @@ int *nbstruct;
 	save_serrno = 0;
 	/* retry as much as the user wishes */
 	while (retrycnt <= nbretry && !isconnected && !abortconnect) {
-
 		if (retrycnt > 0)
 			sleep (retryint);
 		++retrycnt;
 
-		if (neterrstr) {
-			free (neterrstr);
-			neterrstr = NULL;
+		if (aitop) { freeaddrinfo (aitop); aitop = NULL; }
+
+		if (neterrstr) { free (neterrstr); neterrstr = NULL; }
+
+		if (!(aitop = lookup_host (func, dpmhost, strport))) {
+			abortconnect = 1;
+			save_serrno = serrno;
+			continue;
 		}
 
 		for (ai = aitop; ai && !isconnected; ai = ai->ai_next) {
@@ -251,7 +262,8 @@ int *nbstruct;
 #endif
 	}
 
-	freeaddrinfo (aitop);
+	if (aitop) { freeaddrinfo (aitop); aitop = NULL; }
+
 	if (!isconnected) {
 		serrno = (save_serrno) ? save_serrno : SECOMERR;
 		if (!abortconnect) {
