@@ -1,5 +1,5 @@
 /*
- * $Id: dpm_api.h 4360 2011-05-19 11:04:02Z baud $
+ * $Id: dpm_api.h 9744 2013-11-27 15:39:45Z dhsmith $
  */
 
 /*
@@ -8,7 +8,7 @@
  */
 
 /*
- * @(#)$RCSfile: dpm_api.h,v $ $Revision: 4360 $ $Date: 2011-05-19 13:04:02 +0200 (Thu, 19 May 2011) $ CERN IT-GD/CT Jean-Philippe Baud
+ * @(#)$RCSfile: dpm_api.h,v $ $Revision: 9744 $ $Date: 2013-11-27 16:39:45 +0100 (Wed, 27 Nov 2013) $ CERN IT-GD/CT Jean-Philippe Baud
  */
 
 #ifndef _DPM_API_H
@@ -95,6 +95,20 @@ struct dpm_putfilereq {
 	u_signed64	requested_size;
 };
 
+struct dpm_putfilereqx {
+	char		*to_surl;
+	time_t		lifetime;
+	time_t		f_lifetime;
+	char		f_type;
+	char		s_token[CA_MAXDPMTOKENLEN+1];
+	char		ret_policy;
+	char		ac_latency;
+	u_signed64	requested_size;
+	int		reserved; /* must be zero */
+	char		server[CA_MAXHOSTNAMELEN+1];
+	char		pfnhint[CA_MAXSFNLEN+1];
+};
+
 struct dpm_putfilestatus {
 	char		*to_surl;
 	char		*turl;
@@ -145,6 +159,7 @@ struct dpm_tokeninfo {
 
 EXTERN_C int DLL_DECL dpm_abortfiles _PROTO((char *, int, char **, int *, struct dpm_filestatus **));
 EXTERN_C int DLL_DECL dpm_abortreq _PROTO((char *));
+EXTERN_C int DLL_DECL dpm_accessr _PROTO((const char*, int));
 EXTERN_C int DLL_DECL dpm_addfs _PROTO((char *, char *, char *, int, int));
 EXTERN_C int DLL_DECL dpm_addpool _PROTO((struct dpm_pool *));
 EXTERN_C int DLL_DECL dpm_apiinit _PROTO((struct dpm_api_thread_info **));
@@ -182,6 +197,7 @@ EXTERN_C int DLL_DECL dpm_modifyfs _PROTO((char *, char *, int, int));
 EXTERN_C int DLL_DECL dpm_modifypool _PROTO((struct dpm_pool *));
 EXTERN_C int DLL_DECL dpm_ping _PROTO((char *, char *));
 EXTERN_C int DLL_DECL dpm_put _PROTO((int, struct dpm_putfilereq *, int, char **, char *, int, time_t, char *, int *, struct dpm_putfilestatus **));
+EXTERN_C int DLL_DECL dpm_putx _PROTO((int, struct dpm_putfilereqx *, int, char **, char *, int, time_t, char *, int *, struct dpm_putfilestatus **));
 EXTERN_C int DLL_DECL dpm_putdone _PROTO((char *, int, char **, int *, struct dpm_filestatus **));
 EXTERN_C int DLL_DECL dpm_releasespace _PROTO((char *, int));
 EXTERN_C int DLL_DECL dpm_relfiles _PROTO((char *, int, char **, int, int *, struct dpm_filestatus **));
