@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: Cns_main.c,v $ $Revision: 8852 $ $Date: 2013-05-28 11:33:35 +0200 (Tue, 28 May 2013) $ CERN IT-PDP/DM Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: Cns_main.c,v $ $Revision: 10224 $ $Date: 2014-03-06 08:24:35 +0100 (Thu, 06 Mar 2014) $ CERN IT-PDP/DM Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <errno.h>
@@ -58,6 +58,7 @@ char localhost[CA_MAXHOSTNAMELEN+1];
 int maxfds;
 int no_auto_vid = 0;
 int rdonly = 0;
+int secreps = 0;
 int listen_ipv4only=0, listen_ipv6only=0;
 struct Cns_srv_thread_info *Cns_srv_thread_info;
 extern int Cdomainname (char *, int);
@@ -106,9 +107,12 @@ struct main_args *main_args;
 		cmd = main_args->argv[0];
 	Cinitlog (cmd, LOGFILE);
 
+	if (! strcmp (cmd, "dpnsdaemon"))
+		secreps = 1;
+
 	/* process command line options if any */
 
-	while ((c = getopt (main_args->argc, main_args->argv, "c:l:m:nr46t:")) != EOF) {
+	while ((c = getopt (main_args->argc, main_args->argv, "c:l:m:nrs:46t:")) != EOF) {
 		switch (c) {
 		case 'c':
 			nsconfigfile = optarg;
@@ -129,6 +133,18 @@ struct main_args *main_args;
 		case 'r':
 			rdonly++;
 			break;
+		case 's':
+			if (strtol (optarg, &dp, 10))
+				secreps = 1;
+			else
+				secreps = 0;
+			if (*dp != '\0') {
+				nslogit (func,
+				    "Invalid argument for -s option: %s\n",
+				    optarg);
+				exit (USERR);
+			}
+			break;
 		case '4':
 			listen_ipv4only++;
 			break;
@@ -507,6 +523,7 @@ void *arg;
 	const char *clientname;
 	int magic;
 	char *req_data;
+	size_t req_datalen;
 	char reqbuf[REQBUFSZ*2];
 	int req_type = 0;
 	struct Cns_srv_thread_info *thip = (struct Cns_srv_thread_info *) arg;
@@ -520,6 +537,8 @@ void *arg;
 	if (clientname == NULL)
 		clientname = "unknown";
 
+	thip->restrictreps = secreps;
+
 #ifdef CSEC
 	Csec_server_reinitContext (&thip->sec_ctx, CSEC_SERVICE_TYPE_HOST, NULL);
 	if (Csec_server_establishContext (&thip->sec_ctx, thip->s) < 0) {
@@ -534,6 +553,7 @@ void *arg;
 	if (strcmp (thip->Csec_mech, "ID") == 0 ||
 	  Csec_isIdAService (thip->Csec_mech, thip->Csec_auth_id) >= 0) {
 		if (isTrustedHost (thip->s, localhost, localdomain, CNS_SCE, "TRUST")) {
+			thip->restrictreps = 0;
 			if (Csec_server_getAuthorizationId (&thip->sec_ctx,
 			    &thip->Csec_mech, &thip->Csec_auth_id) < 0) {
 				thip->Csec_uid = 0;
@@ -550,7 +570,7 @@ void *arg;
 				return NULL;
 #else
 			} else {	/* mapping will be done later */
-			       thip->Csec_uid = (uid_t) -1;
+				thip->Csec_uid = (uid_t) -1;
 				thip->Csec_gid = (gid_t) -1;
 				alloced_gids = 1;
 #endif
@@ -582,7 +602,7 @@ void *arg;
 
 
 
-	if ((c = getreq (thip->s, &magic, &req_type, &req_data, &clienthost)) == 0) {
+	if ((c = getreq (thip->s, &magic, &req_type, &req_data, &req_datalen, &clienthost)) == 0) {
 
 		/* Get a thip containing a good db connection
 		   Only the dbfd field will be useful */
@@ -604,7 +624,11 @@ void *arg;
 		dbfdok->db_open_done = dbsave.db_open_done;
 		dbfdok->last_db_use = dbsave.last_db_use;
 
-		c = procreq (magic, req_type, req_data, clienthost, dbfdok);
+		c = procreq (magic, req_type, req_data, req_datalen, clienthost, dbfdok);
+		if (dbfdok->dbfd.tr_started) {
+			(void) Cns_end_tr (&dbfdok->dbfd);
+			nslogit (func, "Ended an unexpected database transaction\n");
+		}
 
 		dbsave2 = *dbfdok;
 		*dbfdok = dbsave;
@@ -657,6 +681,7 @@ void *arg;
 	const char *clientname;
 	int magic;
 	char *req_data;
+	size_t req_datalen;
 	char reqbuf[REQBUFSZ-3*LONGSIZE];
 	int req_type = 0;
 	struct Cns_srv_thread_info *thip = (struct Cns_srv_thread_info *) arg;
@@ -667,6 +692,8 @@ void *arg;
 	if (clientname == NULL)
 		clientname = "unknown";
 
+	thip->restrictreps = secreps;
+
 #ifdef CSEC
 	Csec_server_reinitContext (&thip->sec_ctx, CSEC_SERVICE_TYPE_HOST, NULL);
 	if (Csec_server_establishContext (&thip->sec_ctx, thip->s) < 0) {
@@ -681,6 +708,7 @@ void *arg;
 	if (strcmp (thip->Csec_mech, "ID") == 0 ||
 	    Csec_isIdAService (thip->Csec_mech, thip->Csec_auth_id) >= 0) {
 		if (isTrustedHost (thip->s, localhost, localdomain, CNS_SCE, "TRUST")) {
+			thip->restrictreps = 0;
 			if (Csec_server_getAuthorizationId (&thip->sec_ctx,
 			    &thip->Csec_mech, &thip->Csec_auth_id) < 0) {
 				thip->Csec_uid = 0;
@@ -726,8 +754,12 @@ void *arg;
 	}
 #endif
 	req_data = reqbuf;
-	if ((c = getreq (thip->s, &magic, &req_type, &req_data, &clienthost)) == 0) {
-		c = procreq (magic, req_type, req_data, clienthost, thip);
+	if ((c = getreq (thip->s, &magic, &req_type, &req_data, &req_datalen, &clienthost)) == 0) {
+		c = procreq (magic, req_type, req_data, req_datalen, clienthost, thip);
+		if (thip->dbfd.tr_started) {
+			(void) Cns_end_tr (&thip->dbfd);
+			nslogit (func, "Ended an unexpected database transaction\n");
+		}
 		sendrep (thip->s, CNS_RC, c);
 	} else {
 		nslogit (func, "[%s] (%s): Failure getting the request: %s\n",
@@ -745,11 +777,12 @@ void *arg;
 	return (NULL);
 }
 
-getreq(s, magic, req_type, req_data, clienthost)
+getreq(s, magic, req_type, req_data, req_datalen, clienthost)
 int s;
 int *magic;
 int *req_type;
 char **req_data;
+size_t *req_datalen;
 const char **clienthost;
 {
 	struct hostent *hp;
@@ -788,6 +821,7 @@ const char **clienthost;
 					return (SEINTERNAL);
 				}
 			}
+			*req_datalen = l;
 			return (0);
 		}
 		l = n;
@@ -802,10 +836,11 @@ const char **clienthost;
 	return (SEINTERNAL);
 }
 
-procdirreq(magic, req_type, req_data, clienthost, thip)
+procdirreq(magic, req_type, req_data, req_datalen, clienthost, thip)
 int magic;
 int req_type;
 char *req_data;
+size_t req_datalen;
 const char *clienthost;
 struct Cns_srv_thread_info *thip;
 {
@@ -821,6 +856,7 @@ struct Cns_srv_thread_info *thip;
 	fd_set readfd, readmask;
 	struct Cns_file_replica rep_entry;
 	char *req_data2;
+	size_t req_datalen2;
 	struct Cns_seg_metadata smd_entry;
 	DBLISTPTR smdlistptr;
 	struct timeval timeval;
@@ -829,41 +865,41 @@ struct Cns_srv_thread_info *thip;
 	memset (&dblistptr, 0, sizeof(DBLISTPTR));
 	if (req_type == CNS_OPENDIR) {
 		memset (&smdlistptr, 0, sizeof(DBLISTPTR));
-		if (c = Cns_srv_opendir (magic, req_data, clienthost, thip))
+		if (c = Cns_srv_opendir (magic, req_data, req_datalen, clienthost, thip))
 			return (c);
 	} else if (req_type == CNS_LISTCLASS) {
 		memset (&class_entry, 0, sizeof(class_entry));
-		if (c = Cns_srv_listclass (magic, req_data, clienthost, thip,
+		if (c = Cns_srv_listclass (magic, req_data, req_datalen, clienthost, thip,
 		    &class_entry, endlist, &dblistptr))
 			return (c);
 	} else if (req_type == CNS_LISTLINKS) {
 		memset (&lnk_entry, 0, sizeof(lnk_entry));
-		if (c = Cns_srv_listlinks (magic, req_data, clienthost, thip,
+		if (c = Cns_srv_listlinks (magic, req_data, req_datalen, clienthost, thip,
 		    &lnk_entry, endlist, &dblistptr))
 			return (c);
 	} else if (req_type == CNS_LISTREP4GC) {
 		memset (&rep_entry, 0, sizeof(rep_entry));
-		if (c = Cns_srv_listrep4gc (magic, req_data, clienthost, thip,
+		if (c = Cns_srv_listrep4gc (magic, req_data, req_datalen, clienthost, thip,
 		    &rep_entry, endlist, &dblistptr))
 			return (c);
 	} else if (req_type == CNS_LISTREPLICA) {
 		memset (&fmd_entry, 0, sizeof(fmd_entry));
-		if (c = Cns_srv_listreplica (magic, req_data, clienthost, thip,
+		if (c = Cns_srv_listreplica (magic, req_data, req_datalen, clienthost, thip,
 		    &fmd_entry, &rep_entry, endlist, &dblistptr))
 			return (c);
 	} else if (req_type == CNS_LISTREPLICAX) {
 		memset (&rep_entry, 0, sizeof(rep_entry));
-		if (c = Cns_srv_listreplicax (magic, req_data, clienthost, thip,
+		if (c = Cns_srv_listreplicax (magic, req_data, req_datalen, clienthost, thip,
 		    &rep_entry, endlist, &dblistptr))
 			return (c);
 	} else if (req_type == CNS_LISTREPSET) {
 		memset (&rep_entry, 0, sizeof(rep_entry));
-		if (c = Cns_srv_listrepset (magic, req_data, clienthost, thip,
+		if (c = Cns_srv_listrepset (magic, req_data, req_datalen, clienthost, thip,
 		    &rep_entry, endlist, &dblistptr))
 			return (c);
 	} else {
 		memset (&smd_entry, 0, sizeof(smd_entry));
-		if (c = Cns_srv_listtape (magic, req_data, clienthost, thip,
+		if (c = Cns_srv_listtape (magic, req_data, req_datalen, clienthost, thip,
 		    &fmd_entry, &smd_entry, endlist, &dblistptr))
 			return (c);
 	}
@@ -883,49 +919,49 @@ struct Cns_srv_thread_info *thip;
 			endlist = 1;
 		}
 		req_data2  = req_data;
-		if (!endlist && (rc = getreq (thip->s, &magic, &new_req_type, &req_data2, &clienthost)))
+		if (!endlist && (rc = getreq (thip->s, &magic, &new_req_type, &req_data2, &req_datalen2, &clienthost)))
 			endlist = 1;
 		if (req_type == CNS_OPENDIR) {
 			if (new_req_type != CNS_READDIR)
 				endlist = 1;
-			c = Cns_srv_readdir (magic, req_data2, clienthost, thip,
-			    &fmd_entry, &smd_entry, &umd_entry,
+			c = Cns_srv_readdir (magic, req_data2, req_datalen2, clienthost,
+			    thip, &fmd_entry, &smd_entry, &umd_entry,
 			    endlist, &dblistptr, &smdlistptr, &bod);
 		} else if (req_type == CNS_LISTCLASS) {
 			if (new_req_type != CNS_LISTCLASS)
 				endlist = 1;
-			c = Cns_srv_listclass (magic, req_data2, clienthost, thip,
-			    &class_entry, endlist, &dblistptr);
+			c = Cns_srv_listclass (magic, req_data2, req_datalen2, clienthost,
+			    thip, &class_entry, endlist, &dblistptr);
 		} else if (req_type == CNS_LISTLINKS) {
 			if (new_req_type != CNS_LISTLINKS)
 				endlist = 1;
-			c = Cns_srv_listlinks (magic, req_data2, clienthost, thip,
-			    &lnk_entry, endlist, &dblistptr);
+			c = Cns_srv_listlinks (magic, req_data2, req_datalen2, clienthost,
+			    thip, &lnk_entry, endlist, &dblistptr);
 		} else if (req_type == CNS_LISTREP4GC) {
 			if (new_req_type != CNS_LISTREP4GC)
 				endlist = 1;
-			c = Cns_srv_listrep4gc (magic, req_data2, clienthost, thip,
-			    &rep_entry, endlist, &dblistptr);
+			c = Cns_srv_listrep4gc (magic, req_data2, req_datalen2, clienthost,
+			    thip, &rep_entry, endlist, &dblistptr);
 		} else if (req_type == CNS_LISTREPLICA) {
 			if (new_req_type != CNS_LISTREPLICA)
 				endlist = 1;
-			c = Cns_srv_listreplica (magic, req_data2, clienthost, thip,
-			    &fmd_entry, &rep_entry, endlist, &dblistptr);
+			c = Cns_srv_listreplica (magic, req_data2, req_datalen2, clienthost,
+			    thip, &fmd_entry, &rep_entry, endlist, &dblistptr);
 		} else if (req_type == CNS_LISTREPLICAX) {
 			if (new_req_type != CNS_LISTREPLICAX)
 				endlist = 1;
-			c = Cns_srv_listreplicax (magic, req_data2, clienthost, thip,
-			    &rep_entry, endlist, &dblistptr);
+			c = Cns_srv_listreplicax (magic, req_data2, req_datalen2, clienthost,
+			    thip, &rep_entry, endlist, &dblistptr);
 		} else if (req_type == CNS_LISTREPSET) {
 			if (new_req_type != CNS_LISTREPSET)
 				endlist = 1;
-			c = Cns_srv_listrepset (magic, req_data2, clienthost, thip,
-			    &rep_entry, endlist, &dblistptr);
+			c = Cns_srv_listrepset (magic, req_data2, req_datalen2, clienthost,
+			    thip, &rep_entry, endlist, &dblistptr);
 		} else {
 			if (new_req_type != CNS_LISTTAPE)
 				endlist = 1;
-			c = Cns_srv_listtape (magic, req_data2, clienthost, thip,
-			    &fmd_entry, &smd_entry, endlist, &dblistptr);
+			c = Cns_srv_listtape (magic, req_data2, req_datalen2, clienthost,
+			    thip, &fmd_entry, &smd_entry, endlist, &dblistptr);
 		}
 		if (req_data2 != req_data)
 			free (req_data2);
@@ -937,10 +973,11 @@ struct Cns_srv_thread_info *thip;
 	return (rc);
 }
 
-procreq(magic, req_type, req_data, clienthost, thip)
+procreq(magic, req_type, req_data, req_datalen, clienthost, thip)
 int magic;
 int req_type;
 char *req_data;
+size_t req_datalen;
 const char *clienthost;
 struct Cns_srv_thread_info *thip;
 {
@@ -995,268 +1032,268 @@ struct Cns_srv_thread_info *thip;
 #endif
 	switch (req_type) {
 	case CNS_ACCESS:
-		c = Cns_srv_access (magic, req_data, clienthost, thip);
+		c = Cns_srv_access (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_CHCLASS:
-		c = Cns_srv_chclass (magic, req_data, clienthost, thip);
+		c = Cns_srv_chclass (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_CHDIR:
-		c = Cns_srv_chdir (magic, req_data, clienthost, thip);
+		c = Cns_srv_chdir (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_CHMOD:
-		c = Cns_srv_chmod (magic, req_data, clienthost, thip);
+		c = Cns_srv_chmod (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_CHOWN:
-		c = Cns_srv_chown (magic, req_data, clienthost, thip);
+		c = Cns_srv_chown (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_CREAT:
-		c = Cns_srv_creat (magic, req_data, clienthost, thip);
+		c = Cns_srv_creat (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_DELCLASS:
-		c = Cns_srv_deleteclass (magic, req_data, clienthost, thip);
+		c = Cns_srv_deleteclass (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_DELCOMMENT:
-		c = Cns_srv_delcomment (magic, req_data, clienthost, thip);
+		c = Cns_srv_delcomment (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_DELETE:
-		c = Cns_srv_delete (magic, req_data, clienthost, thip);
+		c = Cns_srv_delete (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_ENTCLASS:
-		c = Cns_srv_enterclass (magic, req_data, clienthost, thip);
+		c = Cns_srv_enterclass (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_GETACL:
-		c = Cns_srv_getacl (magic, req_data, clienthost, thip);
+		c = Cns_srv_getacl (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_GETCOMMENT:
-		c = Cns_srv_getcomment (magic, req_data, clienthost, thip);
+		c = Cns_srv_getcomment (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_GETPATH:
-		c = Cns_srv_getpath (magic, req_data, clienthost, thip);
+		c = Cns_srv_getpath (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_GETSEGAT:
-		c = Cns_srv_getsegattrs (magic, req_data, clienthost, thip);
+		c = Cns_srv_getsegattrs (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_LCHOWN:
-		c = Cns_srv_lchown (magic, req_data, clienthost, thip);
+		c = Cns_srv_lchown (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_LISTCLASS:
-		c = procdirreq (magic, req_type, req_data, clienthost, thip);
+		c = procdirreq (magic, req_type, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_LISTTAPE:
-		c = procdirreq (magic, req_type, req_data, clienthost, thip);
+		c = procdirreq (magic, req_type, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_LSTAT:
-		c = Cns_srv_lstat (magic, req_data, clienthost, thip);
+		c = Cns_srv_lstat (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_MKDIR:
-		c = Cns_srv_mkdir (magic, req_data, clienthost, thip);
+		c = Cns_srv_mkdir (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_MODCLASS:
-		c = Cns_srv_modifyclass (magic, req_data, clienthost, thip);
+		c = Cns_srv_modifyclass (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_OPEN:
-		c = Cns_srv_open (magic, req_data, clienthost, thip);
+		c = Cns_srv_open (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_OPENDIR:
-		c = procdirreq (magic, req_type, req_data, clienthost, thip);
+		c = procdirreq (magic, req_type, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_QRYCLASS:
-		c = Cns_srv_queryclass (magic, req_data, clienthost, thip);
+		c = Cns_srv_queryclass (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_READLINK:
-		c = Cns_srv_readlink (magic, req_data, clienthost, thip);
+		c = Cns_srv_readlink (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_RENAME:
-		c = Cns_srv_rename (magic, req_data, clienthost, thip);
+		c = Cns_srv_rename (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_RMDIR:
-		c = Cns_srv_rmdir (magic, req_data, clienthost, thip);
+		c = Cns_srv_rmdir (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_SETACL:
-		c = Cns_srv_setacl (magic, req_data, clienthost, thip);
+		c = Cns_srv_setacl (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_SETATIME:
-		c = Cns_srv_setatime (magic, req_data, clienthost, thip);
+		c = Cns_srv_setatime (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_SETCOMMENT:
-		c = Cns_srv_setcomment (magic, req_data, clienthost, thip);
+		c = Cns_srv_setcomment (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_SETFSIZE:
-		c = Cns_srv_setfsize (magic, req_data, clienthost, thip);
+		c = Cns_srv_setfsize (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_SETSEGAT:
-		c = Cns_srv_setsegattrs (magic, req_data, clienthost, thip);
+		c = Cns_srv_setsegattrs (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_SHUTDOWN:
-		c = Cns_srv_shutdown (magic, req_data, clienthost, thip);
+		c = Cns_srv_shutdown (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_STAT:
-		c = Cns_srv_stat (magic, req_data, clienthost, thip);
+		c = Cns_srv_stat (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_SYMLINK:
-		c = Cns_srv_symlink (magic, req_data, clienthost, thip);
+		c = Cns_srv_symlink (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_UNDELETE:
-		c = Cns_srv_undelete (magic, req_data, clienthost, thip);
+		c = Cns_srv_undelete (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_UNLINK:
-		c = Cns_srv_unlink (magic, req_data, clienthost, thip);
+		c = Cns_srv_unlink (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_UTIME:
-		c = Cns_srv_utime (magic, req_data, clienthost, thip);
+		c = Cns_srv_utime (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_REPLACESEG:
-		c = Cns_srv_replaceseg (magic, req_data, clienthost, thip);
+		c = Cns_srv_replaceseg (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_ADDREPLICA:
-		c = Cns_srv_addreplica (magic, req_data, clienthost, thip);
+		c = Cns_srv_addreplica (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_DELREPLICA:
-		c = Cns_srv_delreplica (magic, req_data, clienthost, thip);
+		c = Cns_srv_delreplica (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_LISTREPLICA:
-		c = procdirreq (magic, req_type, req_data, clienthost, thip);
+		c = procdirreq (magic, req_type, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_STARTTRANS:
-		c = proctransreq (magic, req_data, clienthost, thip);
+		c = proctransreq (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_ENDTRANS:
-		c = Cns_srv_endtrans (magic, req_data, clienthost, thip);
+		c = Cns_srv_endtrans (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_ABORTTRANS:
-		c = Cns_srv_aborttrans (magic, req_data, clienthost, thip);
+		c = Cns_srv_aborttrans (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_LISTLINKS:
-		c = procdirreq (magic, req_type, req_data, clienthost, thip);
+		c = procdirreq (magic, req_type, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_SETFSIZEG:
-		c = Cns_srv_setfsizeg (magic, req_data, clienthost, thip);
+		c = Cns_srv_setfsizeg (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_STATG:
-		c = Cns_srv_statg (magic, req_data, clienthost, thip);
+		c = Cns_srv_statg (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_STATR:
-		c = Cns_srv_statr (magic, req_data, clienthost, thip);
+		c = Cns_srv_statr (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_SETPTIME:
-		c = Cns_srv_setptime (magic, req_data, clienthost, thip);
+		c = Cns_srv_setptime (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_SETRATIME:
-		c = Cns_srv_setratime (magic, req_data, clienthost, thip);
+		c = Cns_srv_setratime (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_SETRSTATUS:
-		c = Cns_srv_setrstatus (magic, req_data, clienthost, thip);
+		c = Cns_srv_setrstatus (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_ACCESSR:
-		c = Cns_srv_accessr (magic, req_data, clienthost, thip);
+		c = Cns_srv_accessr (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_LISTREP4GC:
-		c = procdirreq (magic, req_type, req_data, clienthost, thip);
+		c = procdirreq (magic, req_type, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_LISTREPLICAX:
-		c = procdirreq (magic, req_type, req_data, clienthost, thip);
+		c = procdirreq (magic, req_type, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_STARTSESS:
-		c = procsessreq (magic, req_data, clienthost, thip);
+		c = procsessreq (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_ENDSESS:
-		c = Cns_srv_endsess (magic, req_data, clienthost, thip);
+		c = Cns_srv_endsess (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_DU:
-		c = Cns_srv_du (magic, req_data, clienthost, thip);
+		c = Cns_srv_du (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_GETGRPID:
-		c = Cns_srv_getgrpbynam (magic, req_data, clienthost, thip);
+		c = Cns_srv_getgrpbynam (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_GETGRPNAM:
-		c = Cns_srv_getgrpbygid (magic, req_data, clienthost, thip);
+		c = Cns_srv_getgrpbygid (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_GETIDMAP:
-		c = Cns_srv_getidmap (magic, req_data, clienthost, thip);
+		c = Cns_srv_getidmap (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_GETUSRID:
-		c = Cns_srv_getusrbynam (magic, req_data, clienthost, thip);
+		c = Cns_srv_getusrbynam (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_GETUSRNAM:
-		c = Cns_srv_getusrbyuid (magic, req_data, clienthost, thip);
+		c = Cns_srv_getusrbyuid (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_MODGRPMAP:
-		c = Cns_srv_modgrpmap (magic, req_data, clienthost, thip);
+		c = Cns_srv_modgrpmap (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_MODUSRMAP:
-		c = Cns_srv_modusrmap (magic, req_data, clienthost, thip);
+		c = Cns_srv_modusrmap (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_RMGRPMAP:
-		c = Cns_srv_rmgrpmap (magic, req_data, clienthost, thip);
+		c = Cns_srv_rmgrpmap (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_RMUSRMAP:
-		c = Cns_srv_rmusrmap (magic, req_data, clienthost, thip);
+		c = Cns_srv_rmusrmap (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_GETLINKS:
-		c = Cns_srv_getlinks (magic, req_data, clienthost, thip);
+		c = Cns_srv_getlinks (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_GETREPLICA:
-		c = Cns_srv_getreplica (magic, req_data, clienthost, thip);
+		c = Cns_srv_getreplica (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_ENTGRPMAP:
-		c = Cns_srv_entergrpmap (magic, req_data, clienthost, thip);
+		c = Cns_srv_entergrpmap (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_ENTUSRMAP:
-		c = Cns_srv_enterusrmap (magic, req_data, clienthost, thip);
+		c = Cns_srv_enterusrmap (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_SETRTYPE:
-		c = Cns_srv_setrtype (magic, req_data, clienthost, thip);
+		c = Cns_srv_setrtype (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_MODREPLICA:
-		c = Cns_srv_modreplica (magic, req_type, req_data, clienthost, thip);
+		c = Cns_srv_modreplica (magic, req_type, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_GETREPLICAX:
-		c = Cns_srv_getreplicax (magic, req_data, clienthost, thip);
+		c = Cns_srv_getreplicax (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_LISTREPSET:
-		c = procdirreq (magic, req_type, req_data, clienthost, thip);
+		c = procdirreq (magic, req_type, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_SETRLTIME:
-		c = Cns_srv_setrltime (magic, req_data, clienthost, thip);
+		c = Cns_srv_setrltime (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_GETREPLICAS:
-		c = Cns_srv_getreplicas (magic, req_data, clienthost, thip);
+		c = Cns_srv_getreplicas (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_GETGRPNAMES:
-		c = Cns_srv_getgrpbygids (magic, req_data, clienthost, thip);
+		c = Cns_srv_getgrpbygids (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_PING:
-		c = Cns_srv_ping (magic, req_data, clienthost, thip);
+		c = Cns_srv_ping (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_DELFILES:
-		c = Cns_srv_delfiles (magic, req_data, clienthost, thip);
+		c = Cns_srv_delfiles (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_DELFILESBYP:
-		c = Cns_srv_delfilesbypattern (magic, req_data, clienthost, thip);
+		c = Cns_srv_delfilesbypattern (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_DELREPLICAS:
-		c = Cns_srv_delreplicas (magic, req_data, clienthost, thip);
+		c = Cns_srv_delreplicas (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_GETGRPMAP:
-		c = Cns_srv_getgrpmap (magic, req_data, clienthost, thip);
+		c = Cns_srv_getgrpmap (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_GETUSRMAP:
-		c = Cns_srv_getusrmap (magic, req_data, clienthost, thip);
+		c = Cns_srv_getusrmap (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_GETREPLICAL:
-		c = Cns_srv_getreplicasl (magic, req_data, clienthost, thip);
+		c = Cns_srv_getreplicasl (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_MODREPLICAX:
-		c = Cns_srv_modreplica (magic, req_type, req_data, clienthost, thip);
+		c = Cns_srv_modreplica (magic, req_type, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_DELREPBYSFN:
-		c = Cns_srv_delreplicasbysfn (magic, req_data, clienthost, thip);
+		c = Cns_srv_delreplicasbysfn (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_REGFILES:
-		c = Cns_srv_registerfiles (magic, req_data, clienthost, thip);
+		c = Cns_srv_registerfiles (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	case CNS_GETREPLICASS:
-		c = Cns_srv_getreplicass (magic, req_data, clienthost, thip);
+		c = Cns_srv_getreplicass (magic, req_data, req_datalen, clienthost, thip);
 		break;
 	default:
 		sendrep (thip->s, MSG_ERR, NS003, req_type);
@@ -1265,9 +1302,10 @@ struct Cns_srv_thread_info *thip;
 	return (c);
 }
 
-procsessreq(magic, req_data, clienthost, thip)
+procsessreq(magic, req_data, req_datalen, clienthost, thip)
 int magic;
 char *req_data;
+size_t req_datalen;
 const char *clienthost;
 struct Cns_srv_thread_info *thip;
 {
@@ -1275,9 +1313,10 @@ struct Cns_srv_thread_info *thip;
 	int rc = 0;
 	fd_set readfd, readmask;
 	char *req_data2;
+	size_t req_datalen2;
 	struct timeval timeval;
 
-	(void) Cns_srv_startsess (magic, req_data, clienthost, thip);
+	(void) Cns_srv_startsess (magic, req_data, req_datalen, clienthost, thip);
 	sendrep (thip->s, CNS_IRC, 0);
 
 	/* wait for requests and process them */
@@ -1294,12 +1333,12 @@ struct Cns_srv_thread_info *thip;
 			return (rc);
 		}
 		req_data2 = req_data;
-		if (rc = getreq (thip->s, &magic, &req_type, &req_data2, &clienthost)) {
+		if (rc = getreq (thip->s, &magic, &req_type, &req_data2, &req_datalen2, &clienthost)) {
 			if (req_data2 != req_data)
 				free (req_data2);
 			return (rc);
 		}
-		rc = procreq (magic, req_type, req_data2, clienthost, thip);
+		rc = procreq (magic, req_type, req_data2, req_datalen2, clienthost, thip);
 		if (req_data2 != req_data)
 			free (req_data2);
 		if (req_type == CNS_ENDSESS) break;
@@ -1308,9 +1347,10 @@ struct Cns_srv_thread_info *thip;
 	return (rc);
 }
 
-proctransreq(magic, req_data, clienthost, thip)
+proctransreq(magic, req_data, req_datalen, clienthost, thip)
 int magic;
 char *req_data;
+size_t req_datalen;
 const char *clienthost;
 struct Cns_srv_thread_info *thip;
 {
@@ -1318,9 +1358,10 @@ struct Cns_srv_thread_info *thip;
 	int rc = 0;
 	fd_set readfd, readmask;
 	char *req_data2;
+	size_t req_datalen2;
 	struct timeval timeval;
 
-	(void) Cns_srv_starttrans (magic, req_data, clienthost, thip);
+	(void) Cns_srv_starttrans (magic, req_data, req_datalen, clienthost, thip);
 	sendrep (thip->s, CNS_IRC, 0);
 
 	/* wait for requests and process them */
@@ -1334,17 +1375,17 @@ struct Cns_srv_thread_info *thip;
 		if ((rc = select (thip->s+1, &readfd, (fd_set *)0, (fd_set *)0, &timeval)) <= 0) {
 			rc = rc ? SEINTERNAL : SETIMEDOUT;
 			nslogit (func, NS002, "proctransreq", sstrerror(rc));
-			(void) Cns_srv_aborttrans (magic, req_data, clienthost, thip);
+			(void) Cns_srv_aborttrans (magic, req_data, req_datalen, clienthost, thip);
 			return (rc);
 		}
 		req_data2 = req_data;
-		if (rc = getreq (thip->s, &magic, &req_type, &req_data2, &clienthost)) {
-			(void) Cns_srv_aborttrans (magic, req_data2, clienthost, thip);
+		if (rc = getreq (thip->s, &magic, &req_type, &req_data2, &req_datalen2, &clienthost)) {
+			(void) Cns_srv_aborttrans (magic, req_data2, req_datalen2, clienthost, thip);
 			if (req_data2 != req_data)
 				free (req_data2);
 			return (rc);
 		}
-		rc = procreq (magic, req_type, req_data2, clienthost, thip);
+		rc = procreq (magic, req_type, req_data2, req_datalen2, clienthost, thip);
 		if (req_data2 != req_data)
 			free (req_data2);
 		if (req_type == CNS_ENDTRANS || req_type == CNS_ABORTTRANS) break;
