@@ -13,6 +13,8 @@ static char sccsid[] = "@(#)$RCSfile: nsping.c,v $ $Revision: 1.4 $ $Date: 2008/
 #include <sys/types.h>
 #if defined(_WIN32)
 #include <winsock2.h>
+#else
+#include <unistd.h>
 #endif
 #include "Cns_api.h"
 #include "Cns.h"
