@@ -14,8 +14,12 @@
 # star-accounting [-h] [--help]
 # .. to get the help screen
 #
+#
+#  v1.0.0 initial release
+#  v1.0.1 removed site debug printouts that were screwing up the output
+#
 
-__version__ = "1.0"
+__version__ = "1.0.1"
 __author__  = "Fabrizio Furano"
 
 
@@ -73,9 +77,7 @@ def addrecord(xmlroot, hostname, group, user, site, filecount, resourcecapacityu
     usr = etree.SubElement(subjid, SR+"User")
     usr.text = user
 
-  print site
   if site:
-    print "site:"+site+"-"
     st = etree.SubElement(subjid, SR+"Site")
     st.text = site
 
