@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: Csec_protocol_policy.c,v $ $Revision: 6384 $ $Date: 2012-06-06 16:39:34 +0200 (Wed, 06 Jun 2012) $ IT/ADC/CA Benjamin Couturier";
+static char sccsid[] = "@(#)$RCSfile: Csec_protocol_policy.c,v $ $Revision: 9881 $ $Date: 2013-12-16 08:59:20 +0100 (Mon, 16 Dec 2013) $ IT/ADC/CA Benjamin Couturier";
 #endif
 		
 /*
@@ -349,7 +349,7 @@ int Csec_server_negociate_protocol(int s, int timeout, Csec_context_t *ctx, csec
   }
 
   if (_Csec_recv_token(s, &bigbuf, timeout, &received_token_type)<0) {
-    Csec_errmsg(func, "The client did not send an authentication negociation request");
+    Csec_errmsg(func, "The client did not send an authentication negotiation request");
     return -1;
   }
 
@@ -375,7 +375,7 @@ int Csec_server_negociate_protocol(int s, int timeout, Csec_context_t *ctx, csec
 
   unmarshall_LONG(p, ctx->peer_version);
   if (ctx->peer_version < ctx->min_peer_version) {
-    Csec_errmsg(func, "The client's authentication negociation protocol version "
+    Csec_errmsg(func, "The client's authentication negotiation protocol version "
                       "is %d, we require at least %d", ctx->peer_version,
                       ctx->min_peer_version);
     free(bigbuf.value);
@@ -691,7 +691,7 @@ int Csec_server_negociate_protocol(int s, int timeout, Csec_context_t *ctx, csec
     } else {
       serrno = EINVAL;
     }
-    Csec_errmsg(func,"Could not negociate an authentication method with the client");
+    Csec_errmsg(func,"Could not negotiate an authentication method with the client");
     return -1;
   }
 
@@ -847,7 +847,7 @@ int Csec_client_negociate_protocol(int s, int timeout, Csec_context_t *ctx, Csec
 
   bigbuf.length = 0;
   if (_Csec_recv_token(s, &bigbuf, timeout, &received_token_type)<0) {
-    Csec_errmsg(func, "The server did not reply to our authentication negociation request");
+    Csec_errmsg(func, "The server did not reply to our authentication negotiation request");
     return -1;
   }
 
@@ -873,7 +873,7 @@ int Csec_client_negociate_protocol(int s, int timeout, Csec_context_t *ctx, Csec
 
   unmarshall_LONG(p, ctx->peer_version);
   if (ctx->peer_version < ctx->min_peer_version) {
-    Csec_errmsg(func, "The server's authentication negociation protocol version "
+    Csec_errmsg(func, "The server's authentication negotiation protocol version "
                       "is %d, we require at least %d", ctx->peer_version,
                       ctx->min_peer_version);
     free(bigbuf.value);
@@ -1030,7 +1030,7 @@ int Csec_client_negociate_protocol(int s, int timeout, Csec_context_t *ctx, Csec
     if (failure_reason == ESEC_PROTNOTSUPP) {
       serrno = ESEC_PROTNOTSUPP;
     } else {
-      Csec_errmsg(func,"Could not negociate an authentication method with the server");
+      Csec_errmsg(func,"Could not negotiate an authentication method with the server");
       serrno = EINVAL;
     }
     return -1;
