@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char sccsid[] = "@(#)$RCSfile: srmv2_dirreq.c,v $ $Revision: 4497 $ $Date: 2011-06-20 18:37:21 +0200 (Mon, 20 Jun 2011) $ CERN Jean-Philippe Baud";
+static char sccsid[] = "@(#)$RCSfile: srmv2_dirreq.c,v $ $Revision: 7960 $ $Date: 2012-12-04 09:52:23 +0100 (Tue, 04 Dec 2012) $ CERN Jean-Philippe Baud";
 #endif /* not lint */
 
 #include <sys/stat.h>
@@ -78,7 +78,7 @@ ns1__srmMkdir (struct soap *soap, struct ns1__srmMkdirRequest *req, struct ns1__
 		repp->returnStatus->statusCode = SRM_USCOREINVALID_USCOREPATH;
 		RETURNSC (SOAP_OK, SRM_USCOREINVALID_USCOREPATH);
 	}
-	sprintf (logbuf, "Mkdir %s", req->SURL);
+	snprintf (logbuf, sizeof(logbuf), "Mkdir %s", req->SURL);
 	srm_logreq (func, logbuf);
 
 	Cns_seterrbuf (thip->errbuf, sizeof(thip->errbuf));
@@ -136,7 +136,7 @@ removedir (struct soap *soap, char *dir)
 				dll->next = dlc;
 			dll = dlc;
 		} else {
-			sprintf (fullpath, "%s/%s", dir, dxp->d_name);
+			snprintf (fullpath, sizeof(fullpath), "%s/%s", dir, dxp->d_name);
 			rep_entries = NULL;
 			if (Cns_getreplicax (fullpath, NULL, NULL, &nbreplicas,
 			    &rep_entries) < 0) {
@@ -155,7 +155,7 @@ removedir (struct soap *soap, char *dir)
 	}
 	(void) Cns_closedir (dirp);
 	while (dlf) {
-		sprintf (curdir, "%s/%s", dir, dlf->d_name);
+		snprintf (curdir, sizeof(curdir), "%s/%s", dir, dlf->d_name);
 		(void) removedir (soap, curdir);
 		soap_dealloc (soap, dlf->d_name);
 		dlc = dlf;
@@ -222,7 +222,7 @@ ns1__srmRmdir (struct soap *soap, struct ns1__srmRmdirRequest *req, struct ns1__
 	}
 	if (req->recursive && *req->recursive == true_)
 		rflag++;
-	sprintf (logbuf, "Rmdir %s", req->SURL);
+	snprintf (logbuf, sizeof(logbuf), "Rmdir %s", req->SURL);
 	srm_logreq (func, logbuf);
 
 	Cns_seterrbuf (thip->errbuf, sizeof(thip->errbuf));
@@ -475,7 +475,7 @@ listdir (struct soap *soap, char *dir, int nbfiles, int fflag, int nblevels, int
 			rc = -2;
 			break;
 		}
-		sprintf (path, "%s/%s", dir, dxp->d_name);
+		snprintf (path, sizeof(path), "%s/%s", dir, dxp->d_name);
 		rc = listentry (soap, path, dxp, fflag, nblevels, level, offset, count,
 		    nbentries, repfilep->arrayOfSubPaths->pathDetailArray[i++],
 		    sav_gid, sav_gidstr, sav_uid, sav_uidstr, uid, nbgids, gids);
@@ -508,9 +508,9 @@ listdir (struct soap *soap, char *dir, int nbfiles, int fflag, int nblevels, int
 		return (rc);
 	while (dlf) {
 		if (strcmp (dir, "/"))
-			sprintf (path, "%s/%s", dir, dlf->d_name);
+			snprintf (path, sizeof(path), "%s/%s", dir, dlf->d_name);
 		else
-			sprintf (path, "/%s", dlf->d_name);
+			snprintf (path, sizeof(path), "/%s", dlf->d_name);
 		rc = listdir (soap, path, dlf->dirsize, fflag, nblevels, level + 1,
 		    offset, count, nbentries, dlf->repfilep, sav_gid, sav_gidstr,
 		    sav_uid, sav_uidstr, uid, nbgids, gids);
@@ -811,14 +811,18 @@ ns1__srmLs (struct soap *soap, struct ns1__srmLsRequest *req, struct ns1__srmLsR
 
 	repp->details->__sizepathDetailArray = nbsurls;
 	for (i = 0; i < nbsurls; i++) {
-		sprintf (logbuf, "Ls %s", req->arrayOfSURLs->urlArray[i]);
+		char *path = req->arrayOfSURLs->urlArray[i];
+		if (path != NULL) {
+			snprintf (logbuf, sizeof(logbuf), "Ls %s", path);
+		} else {
+			snprintf (logbuf, sizeof(logbuf), "Ls");
+		}
 		srm_logreq (func, logbuf);
 		if ((repp->details->pathDetailArray[i] =
 		    soap_malloc (soap, sizeof(struct ns1__TMetaDataPathDetail))) == NULL)
 			RETURN (SOAP_EOM);
-		c = listentry (soap, req->arrayOfSURLs->urlArray[i],
-		    NULL, fflag, nblevels, 0, offset, count, &nbentries,
-		    repp->details->pathDetailArray[i],
+		c = listentry (soap, path, NULL, fflag, nblevels, 0, offset,
+		    count, &nbentries, repp->details->pathDetailArray[i],
 		    &sav_gid, sav_gidstr, &sav_uid, sav_uidstr, uid, nbgids, gids);
 		if (c == -2)
 			RETURN (SOAP_EOM);
@@ -936,7 +940,7 @@ ns1__srmMv (struct soap *soap, struct ns1__srmMvRequest *req, struct ns1__srmMvR
 		repp->returnStatus->statusCode = SRM_USCOREINVALID_USCOREPATH;
 		RETURNSC (SOAP_OK, SRM_USCOREINVALID_USCOREPATH);
 	}
-	sprintf (logbuf, "Mv %s %s", req->fromSURL, req->toSURL);
+	snprintf (logbuf, sizeof(logbuf), "Mv %s %s", req->fromSURL, req->toSURL);
 	srm_logreq (func, logbuf);
 
 	Cns_seterrbuf (thip->errbuf, sizeof(thip->errbuf));
@@ -951,7 +955,7 @@ ns1__srmMv (struct soap *soap, struct ns1__srmMvRequest *req, struct ns1__srmMvR
 			repp->returnStatus->statusCode = SRM_USCOREINVALID_USCOREPATH;
 			RETURNSC (SOAP_OK, SRM_USCOREINVALID_USCOREPATH);
 		}
-		sprintf (tgtfile, "%s/%s", sfn2, p);
+		snprintf (tgtfile, sizeof(tgtfile), "%s/%s", sfn2, p);
 		sfn2 = tgtfile;
 	}
 	if (Cns_rename (sfn1, sfn2) < 0) {
